/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.ws.axis2;

import org.ow2.jonas.lib.service.AbsServiceImpl;
import org.ow2.jonas.service.ServiceException;
import org.ow2.jonas.ws.axis2.jaxws.Axis2WSEndpoint;
import org.ow2.jonas.ws.jaxws.IJAXWSService;
import org.ow2.jonas.ws.jaxws.IWebServiceEndpoint;
import org.ow2.jonas.ws.jaxws.PortMetaData;
import org.ow2.jonas.ws.jaxws.WSException;
import org.ow2.jonas.ws.jaxws.util.JAXWSClassUtils;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.war.api.IWarClassMetadata;
import org.ow2.util.ee.metadata.ws.api.struct.IWebServiceMarker;
import org.ow2.util.event.api.IEventService;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import javax.jws.HandlerChain;
import javax.naming.Reference;
import javax.servlet.ServletContext;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
/**
 * The Axis2Service class is used to declare Axis2 as a module in JOnAS.
 *
 * @author xiaoda
 */
public class Axis2Service extends AbsServiceImpl implements IJAXWSService {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Axis2Service.class);

    /**
     * The event service used to register listeners.
     */
    private IEventService eventService;


    @Override
    protected void doStart() throws ServiceException {
        // TODO Auto-generated method stub

    }

    @Override
    protected void doStop() throws ServiceException {
        // TODO Auto-generated method stub

    }

    public Reference createNamingReference(final IJaxwsWebServiceRef serviceRef) {
        return null;

    }

    public IWebServiceEndpoint createPOJOWebServiceEndpoint(final IWarClassMetadata metadata,
                                                            final ClassLoader loader,
                                                            ServletContext servletContext) throws WSException {
        // TODO Auto-generated method stub
        String classname = metadata.getJClass().getName().replace('/', '.');

        Class<?> klass = null;
        try {
            klass = loader.loadClass(classname);
        } catch (ClassNotFoundException e) {
            throw new WSException("Impossible to load the WebService class", e);
        }

        IWebServiceMarker ws = metadata.getWebServiceMarker();

        // FIXME
        // Using annotation only values (simple but not sufficent)
        String wsdlLocation = JAXWSClassUtils.getWsdlLocation(klass);

        if (wsdlLocation == null || "".equals(wsdlLocation)) {
            throw new WSException("The wsdl location must be specified!");
        }

        // find the URL pattern to use

        // FIXME
        // The util class only look for annotation on the given class
        // JAXWS defines precises rules when inheritance comes into play
        // They're not respected in this case
        // BTW, IWSMarker contains XML descriptor values overriding annotation values
        String serviceName = JAXWSClassUtils.getServiceName(klass);

        String pattern = null;
        if (!"".equals(serviceName)) {
            // default pattern : /{service-name}
            pattern = "/" + serviceName;
        } else {
            // else /{class.name}Service
            pattern = "/" + klass.getSimpleName() + "Service";
        }

        // Creates PortMetaData to hold deployment/runtime information
        // about the web service
        PortMetaData pmd = new PortMetaData();

        pmd.setUrlPattern(pattern);
        pmd.setWSDLLocation(wsdlLocation);

        StringBuffer handlerXML = new StringBuffer();
        try {
            HandlerChain hc = klass.getAnnotation(HandlerChain.class);

            if (hc != null) {
                URL handlerURL = klass.getClassLoader().getResource(hc.file());
                BufferedReader br = new BufferedReader(new InputStreamReader(handlerURL.openStream()));

                String temp = null;
                temp = br.readLine();
                while (temp != null) {
                    handlerXML.append(temp);
                    temp = br.readLine();
                }
                pmd.setHandlerXML(handlerXML.toString());
            }

        } catch (IOException e) {
            throw new WSException("Unable to Get the handler description file!", e);
        } finally {
            if(pmd.getHandlerXML() != null) {
                logger.debug("Found handlers in this package!");
            }
        }


        Axis2WSEndpoint endpoint = new Axis2WSEndpoint(klass, IWebServiceEndpoint.EndpointType.POJO, pmd, null);

        try {
            // init axis2
            endpoint.init();
        } catch (Exception e) {
            throw new WSException("Exception occurs when creating POJO endpoint!",e);
        }

        logger.info("Endpoint {0} inited", klass.getName());

        return endpoint;
    }


    public void bindEventService(final IEventService eventService) {
        logger.info("EventService instance {0}", eventService);
        this.eventService = eventService;
    }

    public void unbindEventService(final IEventService eventService) {
        this.eventService = null;
    }
}
