/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.ws.cxf.easybeans;

import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.apache.cxf.jaxws.JAXWSMethodInvoker;
import org.apache.cxf.jaxws.support.JaxWsServiceFactoryBean;
import org.apache.cxf.service.Service;
import org.apache.cxf.service.invoker.Invoker;
import org.apache.cxf.service.invoker.Factory;
import org.apache.cxf.message.Exchange;
import org.ow2.easybeans.container.session.stateless.StatelessSessionFactory;
import org.ow2.easybeans.api.bean.EasyBeansSLSB;
import org.ow2.util.pool.api.PoolException;

/**
 * Extended {@link JaxWsServerFactoryBean} to inhibit the resource injection.
 * This is done because we (EasyBeans) want to control resource injection.
 * @author Guillaume Sauthier
 */
public class EasyBeansJaxWsServerFactoryBean extends JaxWsServerFactoryBean {

    /**
     * EJB Factory.
     */
    private StatelessSessionFactory factory;

    /**
     * Delagating constructor.
     * @param serviceFactory Factory for {@link Service} instances.
     * @param factory EasyBeans EJB Factory
     */
    public EasyBeansJaxWsServerFactoryBean(final JaxWsServiceFactoryBean serviceFactory,
                                           final StatelessSessionFactory factory) {
        super(serviceFactory);
        this.factory = factory;
    }

    /**
     * Overrided with no-op because we want to control injection.
     * @param instance unused
     * @see org.apache.cxf.jaxws.JaxWsServerFactoryBean#injectResources(java.lang.Object)
     */
    @Override
    protected void injectResources(final Object instance) {
        // Do nothing (Don't worry, that's OK :) )
    }

    /**
     * Set our own {@link Invoker}.
     * @see org.apache.cxf.jaxws.JaxWsServerFactoryBean#createInvoker()
     */
    @Override
    protected Invoker createInvoker() {
        return new JAXWSMethodInvoker(new EasyBeansInvokerFactory());
    }

    /**
     * @return the real Bean class.
     * @see org.apache.cxf.frontend.ServerFactoryBean#getServiceBeanClass()
     */
    @Override
    public Class<?> getServiceBeanClass() {
        return this.factory.getBeanClass();
    }

    /**
     * This class implements the Factory concept of CXF, that is
     * nicely mapped on the EasyBeans's Pool concept.
     */
    private class EasyBeansInvokerFactory implements Factory {

        /**
         * Creates the object that will be used for the invoke
         *
         * @param e Exchange session
         * @return a new instance from the pool.
         * @throws Throwable
         */
        public Object create(Exchange e) throws Throwable {
            return factory.getPool().get();
        }

        /**
         * Post invoke, this is called to allow the factory to release
         * the object, store it, etc...
         *
         * @param e Exchange session
         * @param o object created from the create method
         */
        public void release(Exchange e, Object o) {
            try {
                factory.getPool().release((EasyBeansSLSB) o);
            } catch (PoolException e1) {
                // Ignored Exception
            }
        }
    }
}
